<?php
/**
 * Theme Functions
 *
 * @package ellada
 * @author NeuronThemes
 * @since 1.0.0
 */

/**
 * Global Variables
 * 
 * Defining global variables to make
 * usage easier in theme.
 */
define( 'ELLADA_DIR', get_template_directory() );
define( 'ELLADA_URI', get_template_directory_uri() );
define( 'ELLADA_STYLESHEET', get_stylesheet_uri() );
define( 'ELLADA_PLACEHOLDER', get_template_directory_uri() . '/assets/images/placeholder.png' );
define( 'ELLADA_VERSION', wp_get_theme()->get( 'Version' ) );

if ( ! function_exists( 'ellada_init' ) ) {
	/**
	 * Init
	 * 
	 * Global function which adds theme support,
	 * register nav menus and call actions for
	 * different php, js and css files.
	 */
	function ellada_init() {
		/**
		 * Text Domain
		 * 
		 * Makes theme available for translation,
		 * translations can be found in the /languages/ directory.
		 */
		load_theme_textdomain( 'ellada', ELLADA_DIR . '/languages' );

		// Theme Support
		add_theme_support( 'custom-logo' );
		add_theme_support( 'post-thumbnails' );
		add_theme_support( 'automatic-feed-links' );
		add_theme_support( 'title-tag' );

		/**
		 * WooCommerce Theme Support
		 * 
		 * Theme fully supports plugin WooCommerce
		 * also it's features in single product
		 * as zoom, lightbox and slider.
		 */
		if ( class_exists( 'WooCommerce' ) ) {
			add_theme_support( 'woocommerce' );
			add_theme_support( 'wc-product-gallery-slider' );
			add_filter( 'woocommerce_enable_setup_wizard', '__return_false' );
		}

		// Theme actions within init function
		include_once( 'inc/plugins.php' );

		// Enqueues the styles
		add_action( 'wp_enqueue_scripts', 'ellada_external_css' );
		
		// Enqueues the scripts
		add_action( 'wp_enqueue_scripts', 'ellada_external_js' );

		// Registers the sidebars
		add_action( 'widgets_init', 'ellada_sidebars' );

		// Register Menus
		register_nav_menus( [
			'primary' => esc_html__( 'Primary Menu', 'ellada' )
		] );

		// Wizard
		require_once( get_parent_theme_file_path( '/inc/wizard/class-wizard.php' ) );
		require_once( get_parent_theme_file_path( '/inc/wizard/wizard-config.php' ) );
	}
}
add_action( 'after_setup_theme', 'ellada_init' );


if ( ! function_exists( 'ellada_content_width' ) ) {
	/**
	 * Set the content width in pixels, based on the theme's design and stylesheet.
	 *
	 * Priority 0 to make it available to lower priority callbacks.
	 *
	 * @global int $content_width Content width.
	 */
	function ellada_content_width() {
		$GLOBALS['content_width'] = apply_filters( 'ellada_content_width', 1170 );
	}
}
add_action( 'after_setup_theme', 'ellada_content_width', 0 );

if ( ! function_exists( 'ellada_external_css' ) ) {
	/**
	 * External CSS
	 */
	function ellada_external_css() {
		wp_enqueue_style( 'ellada-main-style', ELLADA_STYLESHEET );
		wp_enqueue_style( 'ellada-fonts', ellada_fonts_url(), array(), ELLADA_VERSION );

		/**
		 * Change Site Main Width
		 * 
		 * Only if elementor is activated.
		 * 
		 * @since 1.0.0
		 */
		if ( did_action( 'elementor/loaded' ) ) {
			$kit = \Elementor\Plugin::$instance->kits_manager->get_active_kit();

			// Container Width
			if ( isset( $kit->get_settings( 'container_width' )['size'] ) ) {
				$css = 'body[class*="woocommerce-"]:not([class*="elementor-page-"]) .main { max-width: 1300px; margin-left: auto; margin-right: auto; }';

				wp_add_inline_style( 'ellada-main-style', $css );
			}
		}
	}
}

if ( ! function_exists( 'ellada_external_js' ) ) {
	/**
	 * External Javascript
	 */
	function ellada_external_js() {
		if ( ! is_admin() ) {
			wp_enqueue_script( 'ellada-scripts', ELLADA_URI . '/assets/scripts/ellada.js', array( 'jquery' ), ELLADA_VERSION, TRUE );

			wp_localize_script(
				'ellada-scripts', 'ellada_params', array(
					'ajaxurl' => admin_url( 'admin-ajax.php' ),
				)
			);

			is_singular() ? wp_enqueue_script( 'comment-reply' ) : ' ';
		}
	}
}

if ( ! function_exists( 'ellada_fonts_url' ) ) {
	function ellada_fonts_url() {
		$fonts_url = '';
		$fonts     = array();

		/* translators: If there are characters in your language that are not supported by this font, translate this to 'off'. Do not translate into your own language. */
		if ( 'off' !== esc_html_x( 'on', 'Butler font: on or off', 'ellada' ) ) {
			$fonts[] = 'Butler:300,400,500, 600';
		}

		/* translators: If there are characters in your language that are not supported by this font, translate this to 'off'. Do not translate into your own language. */
		if ( 'off' !== esc_html_x( 'on', 'Inter font: on or off', 'ellada' ) ) {
			$fonts[] = 'Inter:300,400,500,600';
		}

		if ( $fonts ) {
			$fonts_url = add_query_arg( array(
				'family' => implode( '|', $fonts )
			), 'https://fonts.googleapis.com/css' );
		}

		return $fonts_url;
	}
}

if ( ! function_exists( 'ellada_get_suffix' ) ) {
	/**
	 * Suffix
	 */
	function ellada_get_suffix() {
		return ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';
	}
}

if ( ! function_exists( 'ellada_wp_body_open' ) ) {
	/**
	 * Triggered after the opening <body> tag.
	 * WordPress 5.2
	 * 
	 * @since 1.0.0 
	 */
	function ellada_wp_body_open() {
		if ( function_exists( 'wp_body_open' ) ) {
			wp_body_open();
		} else {
			do_action( 'wp_body_open' );
		}
	}
}

if ( ! function_exists( 'ellada_sidebars' ) ) {
	function ellada_sidebars() {
		$ellada_sidebars = [
			[
				'name' => esc_html__( 'Shop Sidebar', 'ellada' ),
				'description' => esc_html__( 'Widgets on this sidebar are displayed in Shop Pages.', 'ellada' ),
				'id' => 'shop-sidebar',
				'condition' => class_exists( 'WooCommerce' )
			],
			[
                'name' => esc_html__( 'Main Sidebar', 'ellada' ),
                'description' => esc_html__( 'Widgets on this sidebar are displayed in Blog Page.', 'ellada' ),
                'id' => 'main-sidebar'
			],
		];
	
		foreach ( $ellada_sidebars as $sidebar ) {
			if ( isset ( $sidebar['condition'] ) && ! $sidebar['condition'] ) {
				continue;
			}

			register_sidebar(
				[
					'name' => esc_html( $sidebar['name'] ),
					'description' => esc_html( $sidebar['description'] ),
					'id' => esc_attr( $sidebar['id'] ),
					'before_widget' => '<div id="%1$s" class="m-ntheme-widget %2$s">',
					'after_widget'  => '</div>',
					'before_title'  => '<h5 class="m-ntheme-widget__title">',
					'after_title'   => '</h5>'
				]
			);
		}
	}
}

if ( ! function_exists( 'ellada_hamburger' ) ) {
	function ellada_hamburger( $class = 'a-nth-hamburger' ) {

		if ( $class ) {
			$class = 'a-nth-hamburger ' . $class;
		}
	?>
			<a href="#" class="<?php echo esc_attr( $class ); ?>">
               <svg xmlns="http://www.w3.org/2000/svg" width="101" height="22" viewBox="0 0 101 22" fill="none"><rect y="0.899902" width="101" height="1.68333" fill="#301F16"/><rect y="19.4165" width="101" height="1.68333" fill="#301F16"/></svg>
            </a>
		<?php
	}
}

add_filter( 'excerpt_more', 'ellada_excerpt_more' );
if ( ! function_exists( 'ellada_excerpt_more' ) ) {
	function ellada_excerpt_more( $more ) {
		return '...';
	}
}

/**
 * Neuron Functions
 * 
 * Extend the functionality
 * of the theme with extra
 * functions and actions.
 * 
 * @since 1.0.0
 */
include( ELLADA_DIR . '/inc/functions/neuron-functions.php' );

/**
 * WooCommerce Functions
 * 
 * Extend the functionality
 * of the woocommerce with extra
 * functions and actions.
 * 
 * @since 1.0.0
 */
include( ELLADA_DIR . '/inc/functions/woocommerce-functions.php' );

/**
 * Plugin Activation Tool
 * 
 * @since 1.0.0
 */
include_once( ELLADA_DIR . '/inc/tgm/class-tgm-plugin-activation.php' );

/**
 * Demo Import
 * 
 * Hook to add demo import
 * files to the neuron core
 * demo importer.
 * 
 * @since 1.0.0
 */
include_once( ELLADA_DIR . '/inc/demo-import.php' );
